'use strict';

const assert = require('chai').assert;

describe('sinon-sandbox/main', () => {
  let wfSinon;
  let obj;

  beforeEach(() => {
    wfSinon = require('../../');

    obj = {
      foo() {
        return 2;
      },
    };
  });

  afterEach(() => {
    wfSinon.restore();
  });

  it('should return an instance of a sandbox', () => {
    assert.isFunction(wfSinon.spy);
    assert.isFunction(wfSinon.stub);
    assert.isFunction(wfSinon.restore);
  });

  it('should allow restoring a stub', () => {
    assert.strictEqual(obj.foo(), 2);
    wfSinon.stub(obj, 'foo').returns(3);
    assert.strictEqual(obj.foo(), 3);
    wfSinon.restore();
    assert.strictEqual(obj.foo(), 2);
  });

  it('should allow creating a stub after restoring', () => {
    wfSinon.stub(obj, 'foo').returns(3);
    wfSinon.restore();
    wfSinon.stub(obj, 'foo').returns(5);
    assert.strictEqual(obj.foo(), 5);
  });

  describe('fake timers', () => {
    it('should not be enabled by default', () => {
      const now = Date.now();
      assert.notEqual(now, 0);
    });

    it('should not be fake after being restored', () => {
      const clock = wfSinon.useFakeTimers();
      clock.restore();
      const now = Date.now();
      assert.notEqual(now, 0);
    });

    it('should enable tick after being restored', (done) => {
      const clock = wfSinon.useFakeTimers();
      clock.tick(4);
      clock.restore();

      setTimeout(done, 0);
    });
  });
});
